from parse_json_file import parse_config_json_file
import ninja_syntax
import os











# parse_config_json_file().start_parse()

# print(os.stat("jack.build"))
print(os.path.getmtime("jack.build"))





